
###
###  Copyright 2000-2006 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Action::File - base class for psgconf file-generator action types
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::File;

use strict;

use File::Basename;
use File::Compare;
use File::Copy;
use File::Path;
use File::stat;
use POSIX;

use Text::Diff ();		# Use () so we do not get diff() redefinition errors

use PSGConf::Action;

use PSGConf::Util;

our @ISA = qw(PSGConf::Action);


###############################################################################
###  constructor
###############################################################################

my %defaults = (
	mode		=> 0644,
	uid		=> 0,
	gid		=> 0,
	backup		=> 1
);

sub new
{
	my ($class, %opts) = @_;

	map {
		$opts{$_} = $defaults{$_}
			if (!exists($opts{$_}));
	} keys %defaults;

	return PSGConf::Action::new($class, %opts);
}


###############################################################################
###  utility function to check whether file changed
###############################################################################

sub _file_changed
{
	my ($self) = @_;

	### check for changes
	if (! -e $self->{name}
	    || compare($self->{name}, $self->{tmpfile}) != 0)
	{
		$self->{changed} = 1;
		return 1;
	}

	return $self->_file_mode_changed();
}

sub _file_mode_changed
{
	my ($self) = @_;

	### check for ownership or permission differences
	$self->{stat} = stat($self->{name});
	if (!defined($self->{stat}))
	{
		warn "\n\t!!! can't stat '$self->{name}': $!\n";
		return -1;
	}

	if (($self->{mode} != -1
	     && ($self->{stat}->mode & 07777) != $self->{mode})
	    || ($self->{uid} != -1 && $self->{stat}->uid != $self->{uid})
	    || ($self->{gid} != -1 && $self->{stat}->gid != $self->{gid}))
	{
		$self->{changed} = 1;
		return 1;
	}

	### no changes needed
	return 0;
}


###############################################################################
###  check method
###############################################################################

sub check
{
	my ($self, $psgconf) = @_;

	### this space intentionally left blank

	return 0;
}


###############################################################################
###  diff method
###############################################################################

sub diff
{
	my ($self, $psgconf) = @_;
	my ($mytgt);

	print "###############################################################################\n";
	print "### DIFF FOR $self->{name}\n";
	if (! -e $self->{name})
	{
		print "###          (original file does not exist)\n";
		$mytgt = '/dev/null';
	}
	else
	{
		$mytgt = $self->{name};
	}
	print "###############################################################################\n";

	if (defined($self->{stat}))
	{
		if ($self->{mode} != -1
		    && ($self->{stat}->mode & 07777) != $self->{mode})
		{
			printf "+ MODE CHANGE: %04o -> %04o\n",
			       $self->{stat}->mode & 07777,
			       $self->{mode};
		}

		if ($self->{uid} != -1 && $self->{stat}->uid != $self->{uid})
		{
			print '+ UID CHANGE: ' . $self->{stat}->uid
			      . ' -> ' . $self->{uid} . "\n";
		}

		if ($self->{gid} != -1 && $self->{stat}->gid != $self->{gid})
		{
			print '+ GID CHANGE: ' . $self->{stat}->gid
			      . ' -> ' . $self->{gid} . "\n";
		}
	}

	print Text::Diff::diff $mytgt, $self->{tmpfile}
		if (-e $self->{tmpfile});

	print "\n";
}


###############################################################################
###  do() method
###############################################################################

sub do
{
	my ($self, $psgconf) = @_;

	### install new file if contents changed
	if (-e $self->{tmpfile})
	{
		### save or remove original file
		if (-e $self->{name}
		    || -l $self->{name})
		{
			###
			### for regular files, don't delete the original,
			### since it might have multiple hard links
			###

			if (!$self->{backup})
			{
				if (! -f $self->{name})
				{
					rmtree($self->{name});
				}
			}
			elsif (! $psgconf->save_file($self->{name}, $self->{backup}))
			{
				return -1;
			}
		}

		### copy new file into place
		mkpath(dirname($self->{name}));
		if (!copy($self->{tmpfile}, $self->{name}))
		{
			warn "\t!!! copy('$self->{tmpfile}', "
				     . "'$self->{name}'): $!\n";
			return -1;
		}
	}

	### check permissions
	if ((!defined($self->{stat})
	     || ($self->{mode} != -1
		 && ($self->{stat}->mode & 07777) != $self->{mode}))
	    && !chmod($self->{mode}, $self->{name}))
	{
		warn "\t!!! chmod(\"$self->{name}\"): $!\n";
		return -1;
	}

	### check ownership
	if ((!defined($self->{stat})
	     || ($self->{uid} != -1 && $self->{stat}->uid != $self->{uid})
	     || ($self->{uid} != -1 && $self->{stat}->gid != $self->{gid}))
	    && !chown($self->{uid}, $self->{gid}, $self->{name}))
	{
		warn "\t!!! chown(\"$self->{name}\"): $!\n";
		return -1;
	}

	### remove tmpfile if applicable
	if (-e $self->{tmpfile}
	    && $psgconf->{rm_tmpfiles}
	    && !unlink($self->{tmpfile}))
	{
		warn "\t!!! unlink('$self->{tmpfile}'): $!\n";
	}

	return 1;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::File - base class for psgconf file-generator action types

=head1 SYNOPSIS

  use PSGConf::Action::File;

  $psgconf->register_actions(
		PSGConf::Action::File->new(
			'name'		=> '/path/to/file',
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::File> module provides a base class for B<PSGConf>
action classes that manipulate a file.

The B<PSGConf::Action::File> class is not intended to be used to directly
instantiate action objects, but it does support a number of methods for
use in subclasses.

The B<PSGConf::Action::File> class is itself derived from the
B<PSGConf::Action> class, but it defines/overrides the following methods:

=over 4

=item _file_changed()

Utility method for use in subclasses' check() method.  Checks to see
whether the existing file differs from the tmpfile then calls
C<_file_mode_changed()>.  Sets the object's I<changed>attribute
and returns the same values as the check() method.

=item _file_mode_changed()

Utility method for use in subclasses' check() method.  Checks to see
if its mode or permissions changed.  Sets the object's I<changed>
attribute and returns the same values as the check() method.

=item check()

Returns 0.  (This method should be replaced by the subclass.)

=item diff()

Display the differences between the tmpfile and the existing file.
Also displays mode and ownership changes.

=item do()

Moves the tmpfile into place, optionally backing up the original file.
Also sets mode and ownership.

=back

In addition to the attributes supported by the B<PSGConf::Action>
class, the B<PSGConf::Action::File> class supports the following
attributes:

=over 4

=item I<mode>

The octal mode of the file.  Defaults to 0644.  Set to -1 to disable
check.

=item I<uid>

The numeric user id of the file.  Defaults to 0.  Set to -1 to disable
check.

=item I<gid>

The numeric group id of the file.  Defaults to 0.  Set to -1 to disable
check.

=item I<backup>

A boolean flag to indicate whether a backup copy of the existing file
will be saved.  If enabled, existing files will be saved under their
original name but with the prefix ".<backupext>".  Default is to
backup.

=back

=head1 BUGS

The class should support multiple instantiations for the same file,
where subsequent instantiations would use the tmpfile from the previous
instantiation instead of the actual original file on the system.

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action>

L<PSGConf::Util>

=cut

